#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <util/delay.h>
#define DEBUG_TEXT
#include "stringlib.h"
#include "hw_eth.h"
#include "Spi.h"
#include "global.h"
#include "Enc28j60.h"
#include "eth_functions.h"
#include "logger.h"
#include "inttostring.h"
#include "uip.h"
#include "uip_arp.h"

static uint8_t g_nPrescaler = 100;
static volatile struct
{
	uint8_t fPeriodic :1;
} g_nFlags =
{ 0 };


void sysinit(void)
{
	
	uart0_init();
//	uart1_init();
//	rs232_puts1("Hallo");
	rs232_puts_P(device_start);
	set_proc_clk(16000000l);
  timecount = 0;
  init_hardware_eth();
  LED_GN(1);
	uip_ipaddr_t IpAddr;

	//SPI Schnitstelle aktivieren
	rs232_puts_P(spi_start);
	SpiInit();

	//Ethernethbaustein aktivieren
	rs232_puts_P(eth_start);
	Enc28j60Init();
	
	//TCP/IP Protokoll - siehe Beispiel uiP
	rs232_puts_P(arp_start);
	uip_arp_init();
	rs232_puts_P(uip_start);
	uip_init();
	
	//Ports ffnen
	eth_listen();
	rs232_puts_P(eth_status_ports);

	//TIMER1
	//CTC; Prescale 256
	TCCR1B = (1<<WGM12)|(4<<CS10);
	//100ms; wird gebraucht um die ARP Tabelle zu aktualisieren
	OCR1A = 6250;						//6250-> alle 100ms CC_INT
	TIMSK1 = (1<<OCIE1A);

	//Standart Netwerkadressen einstellen
  rs232_puts_P(eth_eeprom_start);
	while(!eeprom_is_ready());
	uip_ipaddr(IpAddr, ENC28J60_IP1, ENC28J60_IP2, ENC28J60_IP3, ENC28J60_IP4);									
	uip_sethostaddr(IpAddr);
	uip_ipaddr(IpAddr, ENC28J60_HIP1, ENC28J60_HIP2, ENC28J60_HIP3, ENC28J60_HIP4);
	uip_setdraddr(IpAddr);
	uip_ipaddr(IpAddr, ENC28J60_NM1, ENC28J60_NM2, ENC28J60_NM3, ENC28J60_NM4);
	uip_setnetmask(IpAddr);
  char enc_ip_buff[16];                    
	rs232_puts_P(eth_status);
	rs232_puts_P(eth_status_ip);
	strcpy(enc_ip_buff, "");
	inttostring(ENC28J60_IP1, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_IP2, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_IP3, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_IP4, enc_ip_buff);
	rs232_puts(enc_ip_buff);
	rs232_puts_P(eth_status_hostip); 
	strcpy(enc_ip_buff, "");
	inttostring(ENC28J60_HIP1, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_HIP2, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_HIP3, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_HIP4, enc_ip_buff);
	rs232_puts(enc_ip_buff);
	rs232_puts_P(eth_status_netmask);
	strcpy(enc_ip_buff, "");
	inttostring(ENC28J60_NM1, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_NM2, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_NM3, enc_ip_buff); strcat(enc_ip_buff, ".");
	inttostring(ENC28J60_NM4, enc_ip_buff);
	rs232_puts(enc_ip_buff);
	
	sei();
}




/*********************MAIN*********************/

int main(void)
{
  sysinit();

	while (1)
	{
	uip_len = Enc28j60Receive(uip_buf);
		if (uip_len > 0)
		{
		rs232_puts_P(eth_message_rcv);
			if (((struct uip_eth_hdr *)&uip_buf[0])->type
					== htons(UIP_ETHTYPE_IP))
			{
				uip_arp_ipin();
				uip_input();

				if (uip_len > 0)
				{
					uip_arp_out();
					Enc28j60Transmit(uip_buf, uip_len);
				}

			}
			else if (((struct uip_eth_hdr *)&uip_buf[0])->type
					== htons(UIP_ETHTYPE_ARP))
			{
				uip_arp_arpin();
				if (uip_len > 0)
				{
					it(uip_buf, uip_len);
				}
			}
		}

		if (g_nFlags.fPeriodic)
		{
			cli();
			g_nFlags.fPeriodic = 0;
			sei();

			int i= UIP_CONNS;
			while (i)
			{
				i--;
				uip_periodic(i);
				if (uip_len > 0)
				{
					uip_arp_out();
					Enc28j60Transmit(uip_buf, uip_len);
				}
			}

			g_nPrescaler--;

			if (g_nPrescaler == 0)
			{
				//Every 10 seconds
				uip_arp_timer();
				g_nPrescaler = 100;
			}
		}
	}

	return 0;
}

ISR(TIMER1_COMPA_vect) 				//100ms
{
	g_nFlags.fPeriodic = 1;
	
	static UNS16 cnt,t;
  cnt++;
  if(cnt%10 == 0)
  {
    t=!t;
		LED_GE(t);
  }	
  //key_pressed();
}


ISR(TIMER3_COMPA_vect)				//1ms
{

}